unit MyCommThread;

interface

uses SysUtils, Classes, Rio, Windows, Messages, SoapHTTPClient, InvokeRegistry, OleCtrls, BinIntf,
     Types, SyncObjs, ActiveX;

const WM_UPDATEAVATAR = WM_USER + $1000;

type
  TCommThread = class(TThread)
  private
    { Private declarations }
    UserIsOnline: Boolean;
    MyUserName, Msg, UserToChatWith: String;
  public
    constructor Create(CreateSuspended: Boolean; OnlineUser: Boolean; User, MyUser: String); overload;
    procedure UpdateMessagesWindow;
    procedure UpdateTheAvatar;
    function HasTerminated: Boolean;
    procedure AddMessageToQueue(Msg: String);
  protected
    procedure Execute; override;
  end;

implementation

uses MainForm;

constructor TCommThread.Create(CreateSuspended: Boolean; OnlineUser: Boolean; User, MyUser: String);
begin
  inherited Create(CreateSuspended);
  Priority := tpNormal;
  FreeOnTerminate := True;
  UserIsOnline := OnlineUser;
  UserToChatWith := User;
  MyUserName := MyUser;
end;

procedure TCommThread.AddMessageToQueue(Msg: String);
begin
  CS.Enter;
  (MainClientForm.HTTPRIO1 as ISoapBinary).SendMessage(Msg, MyUserName, UserToChatWith);
  CS.Leave;
end;

procedure TCommThread.UpdateMessagesWindow;
begin
  MainClientForm.AppendToWB(Msg);
end;

procedure TCommThread.UpdateTheAvatar;
begin
  MainClientForm.UpdateAvatar;
end;

procedure TCommThread.Execute;
var
  OldState: Boolean;
  FileData: TByteDynArray;
  List: TStringList;
  I: Integer;
  SS: String;
begin
  CoInitialize(nil);

  while (not Terminated) do
  begin
    { Pinging time }
    Sleep(250);

    { Do we need to get his avatar? }
    CS.Enter;
    OldState := UserIsOnline;
    UserIsOnline := (MainClientForm.HTTPRIO1 as ISoapBinary).IsUserOnline(UserToChatWith);
    if UserIsOnline <> OldState then
    begin
      if UserIsOnline then
       Msg := '<b>['+TimeToStr(Now)+'] User "'+UserToChatWith+'" is now online.</b><br>'
      else
       Msg := '<b><i>['+TimeToStr(Now)+'] User "'+UserToChatWith+'" is now offline.</i></b><br>';
      Synchronize(UpdateMessagesWindow);

      { Retrieve avatar for the other user, if he/she just got online }
      if UserIsOnline then
      begin
        FileData := (MainClientForm.HTTPRIO1 as ISoapBinary).ReceiveAvatar(UserToChatWith);
        if Length(FileData) > 0 then
        begin
          MainClientForm.ByteArrayToFile(FileData, ExtractFilePath(ParamStr(0))+'OtherUserAvatar.jpg');
          Synchronize(UpdateTheAvatar);
        end;
      end;
    end;

    { Let's check if he/she posted any messages for us }
    List := TStringList.Create;
    List.Text := (MainClientForm.HTTPRIO1 as ISoapBinary).ReceiveMessage(MyUserName, UserToChatWith);
    if List.Text <> '' then
    begin
      // let's parse these messages and show them to our user
      for I := 0 to List.Count-1 do
      if List[I] <> '' then
      begin
        SS := ExtractFilePath(ParamStr(0));
        SS := StringReplace(SS, '\', '/', [rfReplaceAll]);
        Msg := '<font color="#a53737"><b>['+TimeToStr(Now)+'] '+UserToChatWith+':</b> '+List[I]+'</font><br>';

        Msg := StringReplace(Msg, ':D', '<img src="'+SS+'/Res/Smile2.jpg">', [rfIgnoreCase, rfReplaceAll]);
        Msg := StringReplace(Msg, ':-D', '<img src="'+SS+'/Res/Smile2.jpg">', [rfIgnoreCase, rfReplaceAll]);

        Msg := StringReplace(Msg, ';)', '<img src="'+SS+'/Res/Nice.jpg">', [rfReplaceAll]);
        Msg := StringReplace(Msg, ';-)', '<img src="'+SS+'/Res/Nice.jpg">', [rfReplaceAll]);

        Msg := StringReplace(Msg, ':(', '<img src="'+SS+'/Res/SadSmile.jpg">', [rfReplaceAll]);
        Msg := StringReplace(Msg, ':-(', '<img src="'+SS+'/Res/SadSmile.jpg">', [rfReplaceAll]);

        Msg := StringReplace(Msg, ':))', '<img src="'+SS+'/Res/HappySmile.jpg">', [rfReplaceAll]);
        Msg := StringReplace(Msg, ':-))', '<img src="'+SS+'/Res/HappySmile.jpg">', [rfReplaceAll]);

        Msg := StringReplace(Msg, ':)', '<img src="'+SS+'/Res/Smile.jpg">', [rfReplaceAll]);
        Msg := StringReplace(Msg, ':-)', '<img src="'+SS+'/Res/Smile.jpg">', [rfReplaceAll]);

        Synchronize(UpdateMessagesWindow);
      end;
    end;
    List.Free;
    CS.Leave;
  end; // while (not Terminated)

  CoUnInitialize;
end;

function TCommThread.HasTerminated: Boolean;
begin
  Result := Terminated;
end;

end.

