unit MainForm;

interface

uses
  Windows, Types, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, Rio, SoapHTTPClient, InvokeRegistry, ExtCtrls,
  ComCtrls, XPMan, OleCtrls, SHDocVw, Buttons, INIFiles, JPEG, MyCommThread,
  SyncObjs, HTMLLite;
  
type
  TMainClientForm = class(TForm)
    OpenDialog: TOpenDialog;
    HTTPRIO1: THTTPRIO;
    SaveDialog1: TSaveDialog;
    XPManifest1: TXPManifest;
    Panel1: TPanel;
    btnSendMessage: TButton;
    Panel2: TPanel;
    Splitter1: TSplitter;
    SpeedButton1: TSpeedButton;
    imgAvatar: TImage;
    Bevel1: TBevel;
    Panel3: TPanel;
    imgOtherUserAvatar: TImage;
    Bevel2: TBevel;
    MessageField: TMemo;
    Panel4: TPanel;
    HTML: ThtmlLite;
    SpeedButton2: TSpeedButton;
    SpeedButton3: TSpeedButton;
    SpeedButton4: TSpeedButton;
    SpeedButton5: TSpeedButton;
    procedure btnSendMessageClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure MessageFieldKeyPress(Sender: TObject; var Key: Char);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure SpeedButton1Click(Sender: TObject);
    procedure SpeedButton2Click(Sender: TObject);
    procedure SpeedButton3Click(Sender: TObject);
    procedure SpeedButton4Click(Sender: TObject);
    procedure SpeedButton5Click(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
    UserName, UserToChatWith: String;
    AvatarImage: String;
    UserIsOnline: Boolean;
    CommThread: TCommThread;
    Conversation: String;
    procedure AppendToWB(const html: widestring) ;
    procedure ByteArrayToFIle(const ByteArray : TByteDynArray;
                              const FileName : string);
    function FileToByteArray(const FileName : string): TByteDynArray;

    procedure UpdateAvatar;
  end;

var
  MainClientForm: TMainClientForm;
  CS: TCriticalSection;

implementation

uses BinIntf, Math, MSHTML;

{$R *.dfm}

{ Convert an array of bytes into a file }
procedure TMainClientForm.ByteArrayToFIle( const ByteArray : TByteDynArray;
                            const FileName : string );
var Count : integer;
    F : FIle of Byte;
    pTemp : Pointer;
begin
  AssignFile(F, FileName);
  Rewrite(F);
  try
    Count := Length(ByteArray);
    pTemp := @ByteArray[0];
    BlockWrite(F, pTemp^, Count);
  finally
    CloseFile(F);
  end;
end;

{ Convert a file to an array of bytes }
function TMainClientForm.FileToByteArray(const FileName: string): TByteDynArray;
const BLOCK_SIZE=1024;
var BytesRead, BytesToWrite, Count : integer;
    F : FIle of Byte;
    pTemp : Pointer;
begin
  AssignFile( F, FileName );
  Reset(F);
  try
    Count := FileSize( F );
    SetLength(Result, Count );
    pTemp := @Result[0];
    BytesRead := BLOCK_SIZE;
    while (BytesRead = BLOCK_SIZE ) do
    begin
      BytesToWrite := Min(Count, BLOCK_SIZE);
      BlockRead(F, pTemp^, BytesToWrite , BytesRead );
      pTemp := Pointer(LongInt(pTemp) + BLOCK_SIZE);
      Count := Count-BytesRead;
    end;
  finally
    CloseFile( F );
  end;
end;

{ Send the message to the user }
procedure TMainClientForm.btnSendMessageClick(Sender: TObject);
var SS, Msg: String;
begin
  if Trim(MessageField.Text) = '' then Exit; // no send

  SS := ExtractFilePath(ParamStr(0));
  SS := StringReplace(SS, '\', '/', [rfReplaceAll]);

  Msg := MessageField.Text;

  Msg := StringReplace(Msg, ':D', '<img src="'+SS+'/Res/Smile2.jpg">', [rfIgnoreCase, rfReplaceAll]);
  Msg := StringReplace(Msg, ':-D', '<img src="'+SS+'/Res/Smile2.jpg">', [rfIgnoreCase, rfReplaceAll]);

  Msg := StringReplace(Msg, ';)', '<img src="'+SS+'/Res/Nice.jpg">', [rfReplaceAll]);
  Msg := StringReplace(Msg, ';-)', '<img src="'+SS+'/Res/Nice.jpg">', [rfReplaceAll]);

  Msg := StringReplace(Msg, ':(', '<img src="'+SS+'/Res/SadSmile.jpg">', [rfReplaceAll]);
  Msg := StringReplace(Msg, ':-(', '<img src="'+SS+'/Res/SadSmile.jpg">', [rfReplaceAll]);

  Msg := StringReplace(Msg, ':))', '<img src="'+SS+'/Res/HappySmile.jpg">', [rfReplaceAll]);
  Msg := StringReplace(Msg, ':-))', '<img src="'+SS+'/Res/HappySmile.jpg">', [rfReplaceAll]);

  Msg := StringReplace(Msg, ':)', '<img src="'+SS+'/Res/Smile.jpg">', [rfReplaceAll]);
  Msg := StringReplace(Msg, ':-)', '<img src="'+SS+'/Res/Smile.jpg">', [rfReplaceAll]);

  AppendToWB('<font color=#808080><b>['+TimeToStr(Now)+'] '+UserName+':</b> '+Msg+'</font><br>');
  CommThread.AddMessageToQueue(MessageField.Text);
  MessageField.Text := '';
  MessageField.SelStart := 0;
end;

procedure TMainClientForm.FormShow(Sender: TObject);
var InputString, URL: String;
    INIFile: TINIFile;
    JPEG: TJPEGImage;
    FileData: TByteDynArray;
    List: TStringList;
begin
  InputString := InputBox('Chat', 'Enter your username to begin chat:', '');
  UserName := InputString;

  { Show a welcome message }
  Conversation := '';
  Conversation := '<html><body><font face="Verdana" size=3><b>Welcome to chat "'+UserName+'"!</b><hr><font face="Tahoma" size=2>';
  HTML.LoadFromString(Conversation+'</font></body></html>', '');
  HTML.VScrollBarPosition := MainClientForm.HTML.VScrollBarRange;

  { Read name to chat with }
  InputString := InputBox('Chat', 'Enter the username to chat with:', '');
  UserToChatWith := InputString;

  { Ask for our avatar image }
  INIFile := TINIFile.Create(ExtractFilePath(ParamStr(0))+'MiniMessenger.ini');
  AvatarImage := INIFile.ReadString('Avatar', 'Image', '');
  URL := INIFile.ReadString('URL', 'URL', '');
  if URL <> '' then
   HTTPRIO1.WSDLLocation := StringReplace(HTTPRIO1.WSDLLocation, 'localhost', URL, [rfReplaceAll, rfIgnoreCase]);

  if (AvatarImage = '') or (not FileExists(AvatarImage)) then
  begin
    if OpenDialog.Execute then
     AvatarImage := OpenDialog.FileName;
  end;

  if AvatarImage <> '' then
  begin
    JPEG := TJPEGImage.Create;
    try
     JPEG.LoadFromFile(AvatarImage);
     imgAvatar.Picture.Assign(JPEG);
    except
    end;
    JPEG.Free;
  end;

  { Register me as user }
  (HTTPRIO1 as ISoapBinary).RegisterMe(UserName);

  if Assigned(imgAvatar) and FileExists(AvatarImage) then
  begin
    { Send this avatar to the server for our user }
    INIFile.WriteString('Avatar', 'Image', AvatarImage);
    FileData := FileToByteArray(AvatarImage);
    (HTTPRIO1 as ISoapBinary).RegisterMyAvatar(UserName, FileData);
  end;
  INIFile.Free;

  { Check if other user is online }
  UserIsOnline := (HTTPRIO1 as ISoapBinary).IsUserOnline(UserToChatWith);
  if not UserIsOnline then
    AppendToWB('<b><i>User "'+UserToChatWith+'" is currently offline! He will receive your messages when he comes online</i></b><br>')
  else
  begin
    { Retrieve avatar for the other user }
    FileData := (HTTPRIO1 as ISoapBinary).ReceiveAvatar(UserToChatWith);
    ByteArrayToFile(FileData, ExtractFilePath(ParamStr(0))+'OtherUserAvatar.jpg');
    JPEG := TJPEGImage.Create;
    try
      JPEG.LoadFromFile(ExtractFilePath(ParamStr(0))+'OtherUserAvatar.jpg');
      imgOtherUserAvatar.Picture.Assign(JPEG);
    except
    end;
    JPEG.Free;
  end;

  { Let's chat now! }
  CommThread := TCommThread.Create(False, UserIsOnline, UserToChatWith, UserName);
end;

procedure TMainClientForm.FormClose(Sender: TObject;
  var Action: TCloseAction);
begin
  { Close thread }
  if Assigned(CommThread) then
  begin
    CommThread.Terminate;
    while (not CommThread.HasTerminated) do ; // do nothing, wait for thread termination
  end;

  (HTTPRIO1 as ISoapBinary).UnRegisterMe(UserName);

  { Close now }
  Action := caFree;
end;

{ Append to our Web Browser }
procedure TMainClientForm.AppendToWB(const html: widestring);
var
   Range: IHTMLTxtRange;
begin
   Conversation := Conversation+HTML;
   MainClientForm.HTML.LoadFromString(Conversation+'</font></body></html>', '');
   MainClientForm.HTML.VScrollBarPosition := MainClientForm.HTML.VScrollBarRange;
end;

{ Update the user's avatar }
procedure TMainClientForm.UpdateAvatar;
var JPEG: TJPEGImage;
begin
  JPEG := TJPEGImage.Create;
  try
    JPEG.LoadFromFile(ExtractFilePath(ParamStr(0))+'OtherUserAvatar.jpg');
    imgOtherUserAvatar.Picture.Assign(JPEG);
  except
    imgOtherUserAvatar.Picture := nil;
  end;
  JPEG.Free;
end;

procedure TMainClientForm.MessageFieldKeyPress(Sender: TObject;
  var Key: Char);
begin
  if Key = #13 then
   btnSendMessageClick(Self);
end;

procedure TMainClientForm.FormCreate(Sender: TObject);
begin
  CS := TCriticalSection.Create;
end;

procedure TMainClientForm.FormDestroy(Sender: TObject);
begin
  CS.Free;
end;

procedure TMainClientForm.SpeedButton1Click(Sender: TObject);
begin
  MessageField.SelText := ':-)';
end;

procedure TMainClientForm.SpeedButton2Click(Sender: TObject);
begin
  MessageField.SelText := ':-(';
end;

procedure TMainClientForm.SpeedButton3Click(Sender: TObject);
begin
  MessageField.SelText := ':-))';
end;

procedure TMainClientForm.SpeedButton4Click(Sender: TObject);
begin
  MessageField.SelText := ';-)';
end;

procedure TMainClientForm.SpeedButton5Click(Sender: TObject);
begin
  MessageField.SelText := ':-D';
end;

end.
