unit BinImpl;

interface

uses SysUtils, InvokeRegistry, Windows, Classes, BinIntf, Types;

type
  TSoapBinary = class( TInvokableClass , ISoapBinary )
  protected
    procedure RegisterMe(const MyUserName: String);  stdcall;
    procedure UnRegisterMe(const MyUserName: String);  stdcall;
    procedure RegisterMyAvatar(const MyUserName: String; const Data: TByteDynArray);  stdcall;
    procedure SendMessage(const MyMessage: String; const FromUser: String; const ToUser: String);  stdcall;
    function IsUserOnline(const UserName: String): Boolean;  stdcall;
    function ReceiveMessage(const ForUser: String; const FromUser: String): String;  stdcall;
    function ReceiveAvatar(const ForUser: String): TByteDynArray;  stdcall;
  public
  end;

implementation

uses WebBrokerSoap, uWeb, uMain;

{ TSoapBinary }

{ Register this user, add him to the list of users online }
procedure TSoapBinary.RegisterMe(const MyUserName: String);
var I: Integer;
begin
//  MainServerForm.Memo.Lines.Add('Register user: '+MyUserName);
  CS.Enter;
  with (GetSoapWebModule as TBinWebModule) do
  begin
    SetLength(Users, Length(Users) + 1);
    I := Length(Users)-1;
    Users[I].Name := MyUserName;
    SetLength(Users[I].Avatar, 0);
    Users[I].ListOfPendingMessages := TStringList.Create;
    Users[I].ListOfUsers := TStringList.Create;
  end;
  CS.Leave;
end;

{ Unregister this user, delete him from the list of online users }
procedure TSoapBinary.UnRegisterMe(const MyUserName: String);
var I, J: Integer;
    Found: Boolean;
begin
//  MainServerForm.Memo.Lines.Add('UnRegister user: '+MyUserName);
  CS.Enter;
  Found := False;
  with (GetSoapWebModule as TBinWebModule) do
  begin
    for I:=0 to Length(Users)-1 do
     if LowerCase(MyUserName) = LowerCase(Users[I].Name) then
      begin
        Found := True;
        Break;
      end;
    if Found then
    begin
      Users[I].ListOfPendingMessages.Free;
      Users[I].ListOfUsers.Free;
      for J:=I to Length(Users)-2 do
       Users[J] := Users[J+1];
      SetLength(Users, Length(Users)-1); // deleted
    end;
  end;
  CS.Leave;
end;

{ Set this user's avatar }
procedure TSoapBinary.RegisterMyAvatar(const MyUserName: String; const Data: TByteDynArray);
var I: Integer;
begin
//  MainServerForm.Memo.Lines.Add('Get avatar for: '+MyUserName);
  CS.Enter;
  with (GetSoapWebModule as TBinWebModule) do
  begin
    for I:=0 to Length(Users)-1 do
    begin
      if LowerCase(MyUserName) = LowerCase(Users[I].Name) then
      begin
        SetLength(Users[I].Avatar, Length(Data));
        Users[I].Avatar := Data;
        Break;
      end;
    end;
  end;
  CS.Leave;
end;

{ Our user sent a new message to someone }
procedure TSoapBinary.SendMessage(const MyMessage: String; const FromUser: String; const ToUser: String);
var I: Integer;
begin
//  MainServerForm.Memo.Lines.Add('SendMessage: "'+MyMessage+'" from "'+FromUser+'" to "'+ToUser+'"');
  CS.Enter;
  with (GetSoapWebModule as TBinWebModule) do
  begin
    for I:=0 to Length(Users)-1 do
    begin
      if LowerCase(FromUser) = LowerCase(Users[I].Name) then
      begin
        Users[I].ListOfPendingMessages.Add(MyMessage);
        Users[I].ListOfUsers.Add(ToUser);
        Break;
      end;
    end;
  end;
  CS.Leave;
end;

{ Is this user online? }
function TSoapBinary.IsUserOnline(const UserName: String): Boolean;
var I: Integer;
begin
  CS.Enter;
  Result := False;
  with (GetSoapWebModule as TBinWebModule) do
  begin
    for I:=0 to Length(Users)-1 do
    begin
      if LowerCase(UserName) = LowerCase(Users[I].Name) then
      begin
        Result := True;
        Break;
      end;
    end;
  end;
  CS.Leave;
end;

{ Receive messages from a certain user }
function TSoapBinary.ReceiveMessage(const ForUser: String; const FromUser: String): String;
var I, J: Integer;
begin
//  MainServerForm.Memo.Lines.Add('ReceiveMessage: from "'+FromUser+'" for "'+ForUser+'"');
  CS.Enter;
  Result := '';
  with (GetSoapWebModule as TBinWebModule) do
  begin
    for I:=0 to Length(Users)-1 do
    begin
      if LowerCase(FromUser) = LowerCase(Users[I].Name) then
      begin
        J := 0;
        while J <= Users[I].ListOfPendingMessages.Count-1 do
        begin
          if LowerCase(Users[I].ListOfUsers[J]) = LowerCase(ForUser) then
          begin
            Result := Result+Users[I].ListOfPendingMessages[J]+#13#10;
            Users[I].ListOfPendingMessages.Delete(J);
            Users[I].ListOfUsers.Delete(J);
          end
          else
           Inc(J);
        end;
        Break;
      end;
    end;
  end;
  if Length(Result) > 1 then // delete ending CRLF
   Delete(Result, Length(Result)-1, 2);
  CS.Leave;
end;

{ Receive this user's avatar }
function TSoapBinary.ReceiveAvatar(const ForUser: String): TByteDynArray;
var I: Integer;
begin
//  MainServerForm.Memo.Lines.Add('ReceiveAvatar: for "'+ForUser+'"');
  CS.Enter;
  SetLength(Result, 0);
  with (GetSoapWebModule as TBinWebModule) do
  begin
    for I:=0 to Length(Users)-1 do
    begin
      if LowerCase(ForUser) = LowerCase(Users[I].Name) then
      begin
        SetLength(Result, Length(Users[I].Avatar));
        Result := Users[I].Avatar;
        Break;
      end;
    end;
  end;
  CS.Leave;
end;

initialization
  InvRegistry.RegisterInvokableClass(TSoapBinary);

end.
